package robocontrol;

import java.io.*;
import java.net.*;
import java.util.*;


/***************************************************************
 *Thread for the communication with the LegoAdapter (server).
 *Waits for receiving Strings and sends them to all JavaAdapters.
 *
 *@see <img src="../../source.gif"><a href="AdapterCommThread.java">java-source-code</a>
 *@see <img src="../../source.gif"><a href="../../index.html" target="_parent">jlegocom help docs</a>
 *
 *@version 0.1, 05/2002
 *@author Gerald Kogler 
****************************************************************/
public class AdapterCommThread extends Thread
{
	static final int ADAPTER_PORT = 6134;
	static final String HOST = "212.31.53.14";		//serveisweb
	//static final String HOST = "127.0.0.1";
	Socket socket;
	PrintStream outStream;
	BufferedReader inStream;
	RobotControl robot;
	boolean running = false;
	private static final String PING_EVT = new String("PING");
	private static final boolean DEBUG = true;


	/*************************************************************************
   *Constructor for the AdapterCommThread.
	 *
   *@param name Name of the thread.
   *@param socket ServerSocket to communicate with the RobotControl.
   *@param robot RobotControl from which the thread is started.
  *************************************************************************/
	public AdapterCommThread(String name, RobotControl robot)
	{
		super(name);
		this.robot = robot;

		if (DEBUG) System.out.println("***********");
		
		try{
			socket = new Socket(HOST, ADAPTER_PORT); 
      outStream = new PrintStream(socket.getOutputStream());
			inStream = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			if (DEBUG) System.out.println(this.getName()+": O ADAPTER connected");

			running = true;
		}
		catch(IOException e){
			if (DEBUG) System.out.println(this.getName()+": X Problem while ADAPTER connected: "+e);
		}
	}

	/*************************************************************************
   *Run-Method of the RobotCommThread.
  *************************************************************************/
	public void run()
	{
		String cmd;
		
		try{
			while(running){
			
				if ((cmd = inStream.readLine()) != null) {
					if (DEBUG) System.out.println("*************");
					if (DEBUG) System.out.println(this.getName()+": READ FROM ADAPTER: <"+cmd+">");
					
					//PROBLEM!! kann erst lesen wenn befehl ausgefhrt!	
					if (cmd.startsWith(PING_EVT)){
						//send reply to legoAdapter
						this.send(PING_EVT);
					}
					else
						robot.doit(cmd);
				}
			}	
		}
		catch(IOException e){
			if (DEBUG) System.out.println(this.getName()+": X broken communication with ADAPTER-> reaccepting matrix");
			running = false;
		}
		finally{
			try{ 
				if (socket != null){
					socket.close();
				}
			}
			catch(IOException e){}
		}
	}


  /*****************************************************************
	 *Close all sockets and streams.
	******************************************************************/
  public void destroy()
	{
		if (DEBUG) System.out.println(this.getName()+": destroyed!");
	
		try{
			socket.close();
			socket = null;
			inStream.close();
			inStream = null;
  		outStream.close();
			outStream = null;
		}
		catch(IOException e){
			if (DEBUG) System.out.println(this.getName()+": "+e);
		}
  }
  /*****************************************************************
  *Sends command to the LegoAdapter (currently not used!)
	*
	*@param cmd String representing command sent to LegoAdapter
	******************************************************************/
  public void send(String cmd)
	{
//    try {
			if (running){
      	this.outStream.println(cmd);
				if (DEBUG) System.out.println(this.getName()+": SENT TO ADAPTER: <"+cmd+">");
			}
			else{
				if (DEBUG) System.out.println(this.getName()+": no connection for sending command!");
			}
//		}
/*
		catch(IOException e){
			System.out.println(this.getName()+": X Send to LegoAdapter failed: "+e);
		}
*/
  }
}
