package robocontrol;

import java.io.*;
import java.util.*;
import rcxdirect.*;

/***************************************************************
 *The class RobotControl is used as devices for controlling a
 *Lego Mindstorms Roboter. It is connected and gets the commands 
 *from the server LegoAdapter.
 *
 *@see <img src="../../source.gif"><a href="RobotControl.java">java-source-code</a>
 *@see <img src="../../source.gif"><a href="../../index.html" target="_parent">jlegocom help docs</a>
 *
 *@version 0.1, 05/2002
 *@author Gerald Kogler 
****************************************************************/
public class RobotControl extends Object
{
	private static final int	ATTAC_TIME = 5,
														WORK_TIME = 100;
	private static final String UP = new String("UP"),
															DOWN = new String("DOWN"),
															DERE = new String("DERE"),
															IZQU = new String("IZQU"),
															STOP = new String("STOP"),
															MOVE = new String("MOVE");
	private AdapterCommThread adapterThread;
	private boolean working = false,
									forOK = true,
									backOK = true;

	/*************************************************************************
   *Constructor for the RobotControl.
  *************************************************************************/
	public RobotControl()
	{
		this.initRobots();
		this.connectAdapter();
	}
		
	/*************************************************************************
   *Inits the Robots.
  *************************************************************************/
	private void initRobots()
	{		
		Port.setName("/dev/ttyS0");
    System.out.println("Battery power: "+ROM.getBatteryPower());
	}
		
	/*************************************************************************
   *Connects to the server LegoAdapter.
  *************************************************************************/
	private void connectAdapter()
	{
		adapterThread = new AdapterCommThread("RobotControl", this);
		adapterThread.start();
	}
	
	/*************************************************************************
   *Executes a command.
	 *
	 *@param cmd String representing a command for the Robot.
  *************************************************************************/
	public void doit(String cmd)
	{
		StringTokenizer tok = new StringTokenizer(cmd);
		String next = tok.nextToken();
		next = tok.nextToken();
		if (next.equals(UP))
			this.goForward();
		else if (next.equals(DOWN))
			this.goBackward();
		else if (next.equals(DERE))
			this.turnRight();
		else if (next.equals(IZQU))
			this.turnLeft();
		else if (next.equals(STOP))
			this.stopRobot();
		else if (next.equals(MOVE))
			this.moveRobot();
		else
			System.out.println("command not available!");
	}
	
	/*************************************************************************
   *Turns the robot right.
  *************************************************************************/
	private void turnRight()
	{
		Motor.A.forward();
		Motor.C.backward();
		
	}
	
	/*************************************************************************
   *Turns the robot left.
  *************************************************************************/
	private void turnLeft()
	{
		Motor.C.forward();
		Motor.A.backward();
	}

	/*************************************************************************
   *Moves the robot forwards.
  *************************************************************************/
	private void goForward()
	{
		Motor.A.forward();
		Motor.C.forward();
	}
	
	/*************************************************************************
   *Moves the robot backwards.
  *************************************************************************/
	private void goBackward()
	{
		Motor.A.backward();
		Motor.C.backward();
	}
		
	/*************************************************************************
   *Stops the robot.
  *************************************************************************/
	private void stopRobot()
	{
		Motor.A.stop();
		Motor.C.stop();
	}			
		
	/*************************************************************************
   *Moves the robot forward for 1 second.
  *************************************************************************/
	private void moveRobot()
	{
		Motor.A.forward();
		this.wait(1000);
		Motor.A.stop();
	}	
			
	/*************************************************************************
   *Gives the robot a pause.
  *************************************************************************/
	private void wait(int pause)
	{
		try{Thread.sleep(pause);}
		catch(InterruptedException e){}
	}
	
	/*************************************************************************
   *Main-program of the RobotControl.
  *************************************************************************/
	public static void main(String[] args)
	{
		RobotControl robot = new RobotControl();
	}
}
